/*
  Liquid War 6 is a unique multiplayer wargame.
  Copyright (C)  2005, 2006, 2007, 2008, 2009, 2010, 2011  Christian Mauduit <ufoot@ufoot.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  

  Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
  Contact author        : ufoot@ufoot.org
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "../../gfx.h"
#include "gl-utils.h"

#define SUB "gfx/gl"
#define CAPTURE "capture"
#define BITMAP "bitmap"

int
mod_gl_utils_path_init (mod_gl_utils_context_t *
			utils_context, int argc, char *argv[])
{
  char *data_root_dir = NULL;
  char *user_dir = NULL;
  int ret = 0;

  data_root_dir = lw6sys_get_data_dir (argc, argv);
  if (data_root_dir)
    {
      utils_context->path.data_dir = lw6sys_path_concat (data_root_dir, SUB);
      LW6SYS_FREE (data_root_dir);
    }

  user_dir = lw6sys_get_user_dir (argc, argv);
  if (user_dir)
    {
      utils_context->path.capture_dir =
	lw6sys_path_concat (user_dir, CAPTURE);
      utils_context->path.bitmap_dir = lw6sys_path_concat (user_dir, BITMAP);
      mod_gl_utils_path_update (utils_context);
      LW6SYS_FREE (user_dir);
    }

  ret = (utils_context->path.data_dir && utils_context->path.capture_dir
	 && utils_context->path.bitmap_dir);

  return ret;
}

int
mod_gl_utils_path_update (mod_gl_utils_context_t * utils_context)
{
  int ret = 0;
  char *frame_str = NULL;

  if (utils_context->path.bitmap_dir)
    {
      if (utils_context->path.bitmap_frame_dir)
	{
	  LW6SYS_FREE (utils_context->path.bitmap_frame_dir);
	}
      frame_str = lw6sys_itoa (utils_context->counter.nb_frames);
      if (frame_str)
	{
	  utils_context->path.bitmap_frame_dir =
	    lw6sys_path_concat (utils_context->path.bitmap_dir, frame_str);
	  LW6SYS_FREE (frame_str);
	  ret = 1;
	}
    }

  return ret;
}

void
mod_gl_utils_path_quit (mod_gl_utils_context_t * utils_context)
{
  if (utils_context->path.data_dir)
    {
      LW6SYS_FREE (utils_context->path.data_dir);
    }
  if (utils_context->path.capture_dir)
    {
      LW6SYS_FREE (utils_context->path.capture_dir);
    }
  if (utils_context->path.bitmap_dir)
    {
      LW6SYS_FREE (utils_context->path.bitmap_dir);
    }
  if (utils_context->path.bitmap_frame_dir)
    {
      LW6SYS_FREE (utils_context->path.bitmap_frame_dir);
    }
}
